%%%%%%%%%%%%%%%%%% Example 3.5 %%%%%%%%%%%%%%%%%%
%   Discrete-Time Control Problems using        %
%       MATLAB and the Control System Toolbox   %
%   by J.H. Chow, D.K. Frederick, & N.W. Chbat  %
%         Brooks/Cole Publishing Company        %
%                September 2002                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   ---- 2-input feedback system ----
%
clear
disp('Example 3.5')
% --- define: sampling period, controller gain, integral time constant ---
Ts = 0.04, Kp = 10, Ki = 1/20
Gp = tf([0.0031 0.003],[1 -1.9 0.905],Ts)   % plant as TF object
H  = tf(0.55,[1 -0.45],Ts)                  % sensor as TF object 
Gc = tf([Kp*Ki*Ts 0],[1 -1],Ts)             % integral controller
disp('******>'), pause 

%------- CL transfer function for reference input -----------
Tref = feedback(Gp*Gc,H)                   
[zTref,pTref,kTref] = zpkdata(Tref,'v')     % zeros, poles, gain
disp('Poles in polar form')
[mag_pTref,theta_pTref] = xy2p(pTref);      % poles in polar form
disp('******>'), pause 

disp('DC gain and step response in reference input')
Tref_gainDC = dcgain(Tref)                  % reference DC gain
dtime = 0:Ts:10;                  % discrete time vector for 10 seconds
yref = step(Tref,dtime);          % response to step in reference input
disp('******>'), pause 

%------- repeat for disturbance input -------------
Tdist = feedback(Gp,Gc*H)      
[zTdist,pTdist,kTdist] = zpkdata(Tdist,'v') % zeros, poles, gain
disp('Poles in polar form')
[mag_pTdist,theta_pTdist] = xy2p(pTdist);   % poles in polar form
disp('DC gain and step response in disturbance input')
Tdist_gainDC = dcgain(Tdist)                % disturbance DC gain
ydist = step(Tdist,dtime);        % response to step in disturbance input         
disp('******>'), pause 

%------ plot step responses at every 10th point ----------
kk = [1:10:10/Ts];   
figure
plot(dtime(kk),yref(kk),'*');grid       % plot reference step response
hold on
plot(dtime(kk),ydist(kk),'o');          % plot disturbance step response
hold off
title('Step responses for Example 3.5')
xlabel('Time (s)')
text(6.5,0.90,'reference')              % add labels to plots
text(3.2,0.45,'disturbance')
%%%%%%%%%%
